@@ -96,13 +96,12 @@ assign("check_scaling",   TRUE,                                       envir = Ro
     "3.2.1" = c("0.2.17", "999.999.999"),
     "3.3.0" = c("0.2.18", "999.999.999"),
     "3.4.0" = c("0.2.18", "999.999.999"),
+    "3.5.0" = c("0.2.19", "999.999.999"),
     stop("New RoBMA version needs to be defined in '.check_BayesTools' function!")
   )
 
-  min_OK <- sum(as.numeric(strsplit(BayesTools_required[1], ".", fixed = TRUE)[[1]]) * c(1e9, 1e6, 1e3)) <=
-    sum(unlist(BayesTools.version) * c(1e9, 1e6, 1e3))
-  max_OK <- sum(as.numeric(strsplit(BayesTools_required[2], ".", fixed = TRUE)[[1]]) * c(1e9, 1e6, 1e3)) >=
-    sum(unlist(BayesTools.version) * c(1e9, 1e6, 1e3))
+  min_OK <- package_version(BayesTools_required[1]) <= BayesTools.version
+  max_OK <- package_version(BayesTools_required[2]) >= BayesTools.version
 
   if(min_OK && max_OK){
     return(invisible(TRUE))
